@@ -28,7 +28,7 @@ except ImportError:
 import gnupg
 
 __author__ = "Vinay Sajip"
-__date__  = "$29-Jan-2017 17:09:59$"
+__date__  = "$06-Jul-2017 15:09:39$"
 
 ALL_TESTS = True
 
@@ -177,7 +177,7 @@ class GPGTestCase(unittest.TestCase):
         if os.path.exists(hd):
             self.assertTrue(os.path.isdir(hd),
                             "Not a directory: %s" % hd)
-            shutil.rmtree(hd)
+            shutil.rmtree(hd, ignore_errors=True)
         prepare_homedir(hd)
         self.homedir = hd
         self.gpg = gpg = gnupg.GPG(gnupghome=hd, gpgbinary=GPGBINARY)
@@ -439,7 +439,6 @@ class GPGTestCase(unittest.TestCase):
         key = self.generate_key("Barbara", "Brown", "beta.com")
         barbara = key.fingerprint
         gpg = self.gpg
-        gpg.encoding = 'latin-1'
         if gnupg._py3k:
             data = 'Hello, André!'
         else:
@@ -471,6 +470,27 @@ class GPGTestCase(unittest.TestCase):
         self.assertEqual(data, str(ddata))
         # Test that you can't encrypt with no recipients
         self.assertRaises(ValueError, self.gpg.encrypt, data, [])
+        # Test extra_args parameter
+        edata = str(gpg.encrypt(data, barbara, extra_args=['-z', '0']))
+        ddata = gpg.decrypt(edata, passphrase='bbrown')
+        self.assertEqual(data.encode('ascii'), ddata.data, 'Round-trip must work')
+        # Test on_data functionality
+
+        chunks = []
+
+        def collector(data):
+            chunks.append(data)
+
+        gpg.on_data = collector
+        edata = str(gpg.encrypt(data, barbara))
+        self.assertTrue(chunks)
+        expected = type(chunks[0])().join(chunks)
+        self.assertEqual(expected.decode('ascii'), edata)
+        chunks = []
+        ddata = gpg.decrypt(edata, passphrase='bbrown')
+        self.assertEqual(data.encode('ascii'), ddata.data, 'Round-trip must work')
+        expected = type(chunks[0])().join(chunks)
+        self.assertEqual(expected.decode('ascii'), data)
 
     def test_import_and_export(self):
         "Test that key import and export works"
@@ -535,7 +555,6 @@ class GPGTestCase(unittest.TestCase):
         "Test that signing and verification works"
         logger.debug("test_signature_verification begins")
         key = self.generate_key("Andrew", "Able", "alpha.com")
-        self.gpg.encoding = 'latin-1'
         if gnupg._py3k:
             data = 'Hello, André!'
         else:
@@ -738,7 +757,7 @@ class GPGTestCase(unittest.TestCase):
             decfname = os.path.join(d, 'decrypted file')
             self.do_file_encryption_and_decryption(encfname, decfname)
         finally:
-            shutil.rmtree(d)
+            shutil.rmtree(d, ignore_errors=True)
         logger.debug("test_filename_with_spaces ends")
 
     #@skipIf(os.name == 'nt', 'Test not suitable for Windows')
@@ -774,7 +793,7 @@ class GPGTestCase(unittest.TestCase):
             files = os.listdir(workdir)
             self.assertEqual(files, ["'ab?'"])
         finally:
-            shutil.rmtree(workdir)
+            shutil.rmtree(workdir, ignore_errors=True)
 
     def disabled_test_signing_with_uid(self):  # pragma: no cover
         "Test that signing with uids works. On hold for now."
@@ -799,7 +818,7 @@ class GPGTestCase(unittest.TestCase):
         replicates the original doctest for import_keys as a regular test.
 
         >>> import shutil
-        >>> shutil.rmtree("keys")
+        >>> shutil.rmtree("keys", ignore_errors=True)
         >>> GPGBINARY = os.environ.get('GPGBINARY', 'gpg')
         >>> gpg = GPG(gpgbinary=GPGBINARY, gnupghome="keys")
         >>> input = gpg.gen_key_input(name_email='user1@test', passphrase='pp1')
